@@ -22,9 +22,14 @@ class PatternRewriter:
     has_erased_matched_operation: bool = field(default=False, init=False)
     """Was the matched operation erased."""
 
-    added_operations: List[Operation] = field(default_factory=list, init=False)
+    added_operations_before: List[Operation] = field(default_factory=list,
+                                                     init=False)
     """The operations added directly before the matched operation."""
 
+    added_operations_after: List[Operation] = field(default_factory=list,
+                                                    init=False)
+    """The operations added directly after the matched operation."""
+
     has_done_action: bool = field(default=False, init=False)
     """Has the rewriter done any action during the current match."""
 
@@ -61,7 +66,22 @@ class PatternRewriter:
             return
         op_idx = block.get_operation_index(self.current_operation)
         block.insert_op(op, op_idx)
-        self.added_operations += op
+        self.added_operations_before += op
+
+    def insert_op_after_matched_op(self, op: Union[Operation,
+                                                   List[Operation]]):
+        """Insert operations after the matched operation."""
+        if self.current_operation.parent is None:
+            raise Exception(
+                "Cannot insert an operation after a toplevel operation.")
+        self.has_done_action = True
+        block = self.current_operation.parent
+        op = op if isinstance(op, list) else [op]
+        if len(op) == 0:
+            return
+        op_idx = block.get_operation_index(self.current_operation)
+        block.insert_op(op, op_idx + 1)
+        self.added_operations_after += op
 
     def insert_op_at_pos(self, op: Union[Operation, List[Operation]],
                          block: Block, pos: int):
@@ -151,7 +171,7 @@ class PatternRewriter:
                             new_ops,
                             new_results,
                             safe_erase=safe_erase)
-        self.added_operations += new_ops
+        self.added_operations_before += new_ops
 
     def replace_op(self,
                    op: Operation,
@@ -240,7 +260,7 @@ class PatternRewriter:
             raise Exception(
                 "Cannot move blocks that are not contained in the matched operation."
             )
-        self.added_operations += block.ops
+        self.added_operations_before += block.ops
         Rewriter.inline_block_before(block, self.current_operation)
 
     def inline_block_before(self, block: Block, op: Operation):
@@ -420,19 +440,23 @@ class PatternRewriteWalker:
         if rewriter.has_done_action:
             # If we produce new operations, we rewrite them recursively if requested
             if self.apply_recursively:
-                return len(rewriter.added_operations) - int(
-                    rewriter.has_erased_matched_operation
-                ) if self.walk_reverse else 0
+                return len(rewriter.added_operations_before) + len(
+                    rewriter.added_operations_after) - int(
+                        rewriter.has_erased_matched_operation
+                    ) if self.walk_reverse else 0
             # Else, we rewrite only their regions if they are supposed to be rewritten after
             else:
                 if not self.walk_regions_first:
-                    for new_op in rewriter.added_operations:
+                    for new_op in rewriter.added_operations_before:
                         self._rewrite_op_regions(new_op)
                     if not rewriter.has_erased_matched_operation:
                         self._rewrite_op_regions(op)
+                    for new_op in rewriter.added_operations_after:
+                        self._rewrite_op_regions(new_op)
                 return -1 if self.walk_reverse else len(
-                    rewriter.added_operations) + int(
-                        not rewriter.has_erased_matched_operation)
+                    rewriter.added_operations_before) + len(
+                        rewriter.added_operations_after) + int(
+                            not rewriter.has_erased_matched_operation)
 
         # Otherwise, we only rewrite the regions of the operation if needed
         if not self.walk_regions_first:
