@@ -11,6 +11,7 @@ indentNumSpaces = 2
 class Printer:
 
     stream: Optional[Any] = field(default=None)
+    print_generic_format: bool = field(default=False)
     print_operand_types: bool = field(default=True)
     print_result_types: bool = field(default=True)
     diagnostic: Diagnostic = field(default_factory=Diagnostic)
@@ -25,8 +26,27 @@ class Printer:
     _next_line_callback: List[Callable[[], None]] = field(default_factory=list,
                                                           init=False)
 
-    def _print(self, text: Any):
-        text = str(text)
+    def print(self, *argv) -> None:
+        for arg in argv:
+            if isinstance(arg, str):
+                self.print_string(arg)
+                continue
+            if isinstance(arg, SSAValue):
+                self.print_ssa_value(arg)
+                continue
+            if isinstance(arg, Attribute):
+                self.print_attribute(arg)
+                continue
+            if isinstance(arg, Region):
+                self.print_region(arg)
+                continue
+            if isinstance(arg, Block):
+                self.print_block_name(arg)
+                continue
+            text = str(arg)
+            self.print_string(text)
+
+    def print_string(self, text) -> None:
         lines = text.split('\n')
         if len(lines) != 1:
             self._current_line += len(lines) - 1
@@ -35,9 +55,6 @@ class Printer:
             self._current_column += len(lines[-1])
         print(text, end='', file=self.stream)
 
-    def print_string(self, string) -> None:
-        self._print(string)
-
     def _add_message_on_next_line(self, message: str, begin_pos: int,
                                   end_pos: int):
         """Add a message that will be displayed on the next line."""
@@ -55,20 +72,20 @@ class Printer:
         This is expected to be called on the beginning of a new line, and expect to create a new line at the end.
         """
         indent = self._indent if indent is None else indent
-        self._print(" " * indent * indentNumSpaces)
+        self.print(" " * indent * indentNumSpaces)
         indent_size = indent * indentNumSpaces
         message_end_pos = max([len(line) for line in message.split("\n")]) + 2
         first_line = (begin_pos - indent_size) * "-" + (
             end_pos - begin_pos + 1) * "^" + (max(message_end_pos, end_pos) -
                                               end_pos) * "-"
-        self._print(first_line)
+        self.print(first_line)
         self._print_new_line(indent=indent, print_message=False)
         message_lines = message.split("\n")
         for message_line in message_lines:
-            self._print("| ")
-            self._print(message_line)
+            self.print("| ")
+            self.print(message_line)
             self._print_new_line(indent=indent, print_message=False)
-        self._print("-" * (max(message_end_pos, end_pos) - indent_size + 1))
+        self.print("-" * (max(message_end_pos, end_pos) - indent_size + 1))
         self._print_new_line(indent=0, print_message=False)
 
     T = TypeVar('T')
@@ -79,18 +96,18 @@ class Printer:
             return
         print_fn(elems[0])
         for elem in elems[1:]:
-            self._print(", ")
+            self.print(", ")
             print_fn(elem)
 
     def _print_new_line(self, indent=None, print_message=True) -> None:
         indent = self._indent if indent is None else indent
-        self._print("\n")
+        self.print("\n")
         if print_message:
             while len(self._next_line_callback) != 0:
                 callback = self._next_line_callback[0]
                 self._next_line_callback = self._next_line_callback[1:]
                 callback()
-        self._print(" " * indent * indentNumSpaces)
+        self.print(" " * indent * indentNumSpaces)
 
     def _get_new_valid_name_id(self) -> str:
         self._next_valid_name_id += 1
@@ -102,7 +119,7 @@ class Printer:
 
     def _print_result_value(self, op: Operation, idx: int) -> None:
         val = op.results[idx]
-        self._print("%")
+        self.print("%")
         if val in self._ssa_values.keys():
             name = self._ssa_values[val]
         elif val.name:
@@ -113,9 +130,9 @@ class Printer:
         else:
             name = self._get_new_valid_name_id()
             self._ssa_values[val] = name
-        self._print("%s" % name)
+        self.print("%s" % name)
         if self.print_result_types:
-            self._print(" : ")
+            self.print(" : ")
             self.print_attribute(val.typ)
 
     def _print_results(self, op: Operation) -> None:
@@ -127,28 +144,28 @@ class Printer:
         # One result
         if len(results) == 1:
             self._print_result_value(op, 0)
-            self._print(" = ")
+            self.print(" = ")
             return
 
         # Multiple results
-        self._print("(")
+        self.print("(")
         self._print_result_value(op, 0)
         for idx in range(1, len(results)):
-            self._print(", ")
+            self.print(", ")
             self._print_result_value(op, idx)
-        self._print(") = ")
+        self.print(") = ")
+
+    def print_ssa_value(self, value: SSAValue) -> None:
+        if (self._ssa_values.get(value) == None):
+            raise KeyError("SSAValue is not part of the IR, are you sure"
+                           " all operations are added before their uses?")
+        self.print(f"%{self._ssa_values[value]}")
 
     def _print_operand(self, operand: SSAValue) -> None:
-        if (self._ssa_values.get(operand) == None):
-            raise KeyError(
-                "SSAValue is not part of the IR, are you sure all operations are added before their uses?"
-            )
-        self._print("%")
-
-        self._print("%s" % self._ssa_values[operand])
+        self.print_ssa_value(operand)
 
         if self.print_operand_types:
-            self._print(" : ")
+            self.print(" : ")
             self.print_attribute(operand.typ)
 
     def _print_ops(self, ops: List[Operation]) -> None:
@@ -161,84 +178,84 @@ class Printer:
             self._print_new_line()
 
     def print_block_name(self, block: Block) -> None:
-        self._print("^")
+        self.print("^")
         if block not in self._block_names:
             self._block_names[block] = self._get_new_valid_block_id()
-        self._print(self._block_names[block])
+        self.print(self._block_names[block])
 
     def _print_named_block(self, block: Block) -> None:
         self.print_block_name(block)
         if len(block.args) > 0:
-            self._print("(")
+            self.print("(")
             self.print_list(block.args, self._print_block_arg)
-            self._print(")")
-        self._print(":")
+            self.print(")")
+        self.print(":")
         if len(block.ops) > 0:
             self._print_ops(block.ops)
         else:
             self._print_new_line()
 
     def _print_block_arg(self, arg: BlockArgument) -> None:
-        self._print("%")
+        self.print("%")
         name = self._get_new_valid_name_id()
         self._ssa_values[arg] = name
-        self._print("%s : " % name)
+        self.print("%s : " % name)
         self.print_attribute(arg.typ)
 
-    def _print_region(self, region: Region) -> None:
+    def print_region(self, region: Region) -> None:
         if len(region.blocks) == 0:
-            self._print(" {}")
+            self.print(" {}")
             return
 
         if len(region.blocks) == 1 and len(region.blocks[0].args) == 0:
-            self._print(" {")
+            self.print(" {")
             self._print_ops(region.blocks[0].ops)
-            self._print("}")
+            self.print("}")
             return
 
-        self._print(" {")
+        self.print(" {")
         self._print_new_line()
         for block in region.blocks:
             self._print_named_block(block)
-        self._print("}")
+        self.print("}")
 
-    def _print_regions(self, regions: List[Region]) -> None:
+    def print_regions(self, regions: List[Region]) -> None:
         for region in regions:
-            self._print_region(region)
+            self.print_region(region)
 
-    def _print_operands(self, operands: List[SSAValue]) -> None:
+    def _print_operands(self, operands: FrozenList[SSAValue]) -> None:
         if len(operands) == 0:
-            self._print("()")
+            self.print("()")
             return
 
-        self._print("(")
+        self.print("(")
         self._print_operand(operands[0])
         for operand in operands[1:]:
-            self._print(", ")
+            self.print(", ")
             self._print_operand(operand)
-        self._print(")")
+        self.print(")")
 
     def print_attribute(self, attribute: Attribute) -> None:
         if isinstance(attribute, IntegerType):
             width = attribute.parameters[0]
             assert isinstance(width, IntAttr)
-            self._print(f'!i{width.data}')
+            self.print(f'!i{width.data}')
             return
 
         if isinstance(attribute, StringAttr):
-            self._print(f'"{attribute.data}"')
+            self.print(f'"{attribute.data}"')
             return
 
         if isinstance(attribute, FlatSymbolRefAttr):
-            self._print(f'@{attribute.parameters[0].data}')
+            self.print(f'@{attribute.parameters[0].data}')
             return
 
         if isinstance(attribute, IntegerAttr):
             width = attribute.parameters[0]
             typ = attribute.parameters[1]
             assert (isinstance(width, IntAttr))
-            self._print(width.data)
-            self._print(" : ")
+            self.print(width.data)
+            self.print(" : ")
             self.print_attribute(typ)
             return
 
@@ -249,54 +266,63 @@ class Printer:
             return
 
         if isinstance(attribute, Data):
-            self._print(f'!{attribute.name}<')
+            self.print(f'!{attribute.name}<')
             attribute.print(self)
-            self._print(">")
+            self.print(">")
             return
 
         assert isinstance(attribute, ParametrizedAttribute)
 
-        self._print(f'!{attribute.name}')
+        self.print(f'!{attribute.name}')
         if len(attribute.parameters) != 0:
-            self._print("<")
+            self.print("<")
             self.print_list(attribute.parameters, self.print_attribute)
-            self._print(">")
+            self.print(">")
 
     def print_successors(self, successors: List[Block]):
         if len(successors) == 0:
             return
-        self._print(" (")
+        self.print(" (")
         self.print_list(successors, self.print_block_name)
-        self._print(")")
+        self.print(")")
 
     def _print_op_attributes(self, attributes: Dict[str, Attribute]) -> None:
         if len(attributes) == 0:
             return
 
-        self._print(" ")
-        self._print("[")
+        self.print(" ")
+        self.print("[")
 
         attribute_list = [p for p in attributes.items()]
-        self._print("\"%s\" = " % attribute_list[0][0])
+        self.print("\"%s\" = " % attribute_list[0][0])
         self.print_attribute(attribute_list[0][1])
         for (attr_name, attr) in attribute_list[1:]:
-            self._print(", \"%s\" = " % attr_name)
+            self.print(", \"%s\" = " % attr_name)
             self.print_attribute(attr)
-        self._print("]")
+        self.print("]")
+
+    def print_op_with_default_format(self, op: Operation) -> None:
+        self._print_operands(op.operands)
+        self.print_successors(op.successors)
+        self._print_op_attributes(op.attributes)
+        self.print_regions(op.regions)
 
     def _print_op(self, op: Operation) -> None:
         begin_op_pos = self._current_column
         self._print_results(op)
-        self._print(op.name)
-        self._print_operands(op.operands)
-        self.print_successors(op.successors)
-        self._print_op_attributes(op.attributes)
+        if self.print_generic_format:
+            self.print(f'"{op.name}"')
+        else:
+            self.print(op.name)
         end_op_pos = self._current_column - 1
         if op in self.diagnostic.op_messages:
             for message in self.diagnostic.op_messages[op]:
                 self._add_message_on_next_line(message, begin_op_pos,
                                                end_op_pos)
-        self._print_regions(op.regions)
+        if self.print_generic_format:
+            self.print_op_with_default_format(op)
+        else:
+            op.print(self)
 
     def print_op(self, op: Operation) -> None:
         self._print_op(op)
